ManureLagerPlaceable = {};
ManureLagerPlaceable_mt = Class(ManureLagerPlaceable, Placeable);
InitObjectClass(ManureLagerPlaceable, "ManureLagerPlaceable");
ManureLagerPlaceable.sendNumBits = 6
g_ManureLagerPlaceableDir = g_currentModDirectory;

function ManureLagerPlaceable:new(isServer, isClient, customMt)

  local mt = customMt
  if mt == nil then
    mt = ManureLagerPlaceable_mt
  end
    local self = Placeable:new(isServer, isClient, mt);
    registerObjectClassName(self, "ManureLagerPlaceable");
	self.fillLevel = 0;
	self.capacity = 0;
	self.fillTypes = {};
	self.vehiclesInRange = {};
	self.playerInRange = false;
	self.currentFillType = FruitUtil.FILLTYPE_UNKNOWN;
	self.sentcurrentFillType = self.currentFillType
	self.sentFillLevel = self.fillLevel
	self.ManureLagerPlaceableDirtyFlag = self:getNextDirtyFlag();
	return self;
end;

function ManureLagerPlaceable:delete()
    removeTrigger(self.manureTrigger);
    unregisterObjectClassName(self);
		if self.mapHotspot ~= nil then
		g_currentMission.missionPDA:deleteMapHotspot(self.mapHotspot);
	end;
    ManureLagerPlaceable:superClass().delete(self);
end;
function ManureLagerPlaceable:readStream(streamId, connection)
	ManureLagerPlaceable:superClass().readStream(self, streamId, connection)
	if connection:getIsServer() then
		local fillLevel = streamReadFloat32(streamId)
		local fillType = streamReadUIntN(streamId, ManureLagerPlaceable.sendNumBits)
		self:setFillLevel(fillLevel, fillType)
	end;
end;

function ManureLagerPlaceable:writeStream(streamId, connection)
  ManureLagerPlaceable:superClass().writeStream(self, streamId, connection)
  if not connection:getIsServer() then
		streamWriteFloat32(streamId, self.fillLevel)
		streamWriteUIntN(streamId, self.currentFillType, ManureLagerPlaceable.sendNumBits)
	end;
end;

function ManureLagerPlaceable:readUpdateStream(streamId, timestamp, connection)
	ManureLagerPlaceable:superClass().readUpdateStream(self, streamId, timestamp, connection)
	if connection:getIsServer() then
		local fillLevel = streamReadFloat32(streamId)
		local fillType = streamReadUIntN(streamId, ManureLagerPlaceable.sendNumBits)
		self:setFillLevel(fillLevel, fillType)
	end;
end;

function ManureLagerPlaceable:writeUpdateStream(streamId, connection, dirtyMask)
	ManureLagerPlaceable:superClass().writeUpdateStream(self, streamId, connection, dirtyMask)
	if not connection:getIsServer() then
		streamWriteFloat32(streamId, self.fillLevel)
		streamWriteUIntN(streamId, self.currentFillType, ManureLagerPlaceable.sendNumBits)
	end;
end;
function ManureLagerPlaceable:deleteFinal()
    ManureLagerPlaceable:superClass().deleteFinal(self);
end;
function ManureLagerPlaceable:load(xmlFilename, x,y,z, rx,ry,rz, moveMode, initRandom)
    if not ManureLagerPlaceable:superClass().load(self, xmlFilename, x,y,z, rx,ry,rz, moveMode, initRandom) then
        return false;
    end;
	local xmlFile = loadXMLFile("TempXML", xmlFilename);
	self.manureTrigger = Utils.indexToObject(self.nodeId, getXMLString(xmlFile, "placeable.manurelager#triggerIndex"));
	addTrigger(self.manureTrigger, "triggerCallback", self);
	self.capacity = Utils.getNoNil(getXMLFloat(xmlFile, "placeable.manurelager#capacity"), 800000)
	local minY, maxY =  Utils.getVectorFromString(getXMLString(xmlFile, "placeable.manurelager#moveMinMaxY"));
	if minY ~= nil and maxY ~= nil then
		self.moveMinY = minY
		self.moveMaxY = maxY
		self.movingId = Utils.indexToObject(self.nodeId, getXMLString(xmlFile, "placeable.manurelager#movingIndex"));
	end
	self.fillTypes = {};
	self.fillTypes[Fillable.FILLTYPE_UNKNOWN] = true;
	local fillTypes = getXMLString(xmlFile, "placeable.manurelager#fillType");
	if fillTypes ~= nil then
		local types = Utils.splitString(" ", fillTypes);
		for k,v in pairs(types) do
			local fillType = Fillable.fillTypeNameToInt[v];
			if fillType ~= nil then
				self.fillTypes[fillType] = true;
			else
				print("Warning: '"..self.configFileName.. "' has invalid fillType '"..v.."'.");
			end;
		end;
	end;	
delete(xmlFile)
	if not moveMode then
			end
    return true;
end;

function ManureLagerPlaceable:loadFromAttributesAndNodes(xmlFile, key, resetVehicles)
	if not ManureLagerPlaceable:superClass().loadFromAttributesAndNodes(self, xmlFile, key, resetVehicles) then
		return false
	end
	local fillLevel = getXMLFloat(xmlFile, key.."#fillLevel");
	local fillType = getXMLString(xmlFile, key.."#fillType");
	if fillLevel ~= nil and fillType ~= nil then
		local fillTypeInt = Fillable.fillTypeNameToInt[fillType];
		if fillTypeInt ~= nil then
			self:setFillLevel(fillLevel, fillTypeInt);
		end;
	end;

	return true
end
function ManureLagerPlaceable:getSaveAttributesAndNodes(nodeIdent)
  local attributes, nodes = ManureLagerPlaceable:superClass().getSaveAttributesAndNodes(self, nodeIdent)
	local fillType = Fillable.fillTypeIntToName[self.currentFillType];
	if fillType == nil then
		fillType = "unknown";
	end;
	attributes = attributes .. " fillType=\"" .. fillType .. "\""
	attributes = attributes .. " fillLevel=\"" .. self.fillLevel .. "\""
	return attributes, nodes
end

function ManureLagerPlaceable:update(dt)
	if self:getShowInfo() then
		g_currentMission:addExtraPrintText(g_i18n:getText("fill_level") .. " " .. math.floor(self.fillLevel) .. " (" .. math.floor(100 * self.fillLevel / self.capacity) .. "%)")
	end	
end

function ManureLagerPlaceable:updateTick(dt)	
	if self.isServer then
		if self.fillLevel ~= self.sentFillLevel or self.currentFillType ~= self.sentcurrentFillType then
			self:raiseDirtyFlags(self.ManureLagerPlaceableDirtyFlag)
			self.sentFillLevel = self.fillLevel
			self.sentcurrentFillType = self.currentFillType
		end
	end
end
function ManureLagerPlaceable:setFillLevel(fillLevel, fillType, force)
	self.currentFillType = fillType;
	self.fillLevel = fillLevel;
	if self.fillLevel >= self.capacity then
		self.fillLevel = self.capacity;
	end;
	if self.fillLevel <= 0 then
		self.fillLevel = 0;
		self.currentFillType = FruitUtil.FRUITTYPE_UNKNOWN;
	end;
	if self.movingId ~= nil then
		local x, y, z = getTranslation(self.movingId)
		local y = self.moveMinY + (self.moveMaxY - self.moveMinY) * self.fillLevel / self.capacity
		setTranslation(self.movingId, x, y, z)
	end
end;

function ManureLagerPlaceable:DoReFill(sprayer, delta)
	if self.fillLevel < self.capacity then
		sprayer:setFillLevel(sprayer.fillLevel - delta, sprayer.currentFillType, true);
		if sprayer.fillLevel > 0 then
			self:setFillLevel(self.fillLevel + delta, sprayer.currentFillType, true);
		end
	end;
end

function ManureLagerPlaceable:getShowInfo()
	if (g_currentMission.controlPlayer and self.playerInRange) then
		return true;
	end;
	if not g_currentMission.controlPlayer then
		for vehicle in pairs(self.vehiclesInRange) do
			if vehicle:getIsActiveForInput(false) then
				return true;
			end;
		end;
	end;
	return false;
end;
function ManureLagerPlaceable:getIsActivatable2(fillable)
	return true
end

function ManureLagerPlaceable:fill(sprayer, delta)
	if not sprayer:allowFillType(self.currentFillType, false) then
		return 0
	end
	delta = math.max(math.min(delta, self.fillLevel), 0)
	local oldFillLevel = sprayer:getFillLevel(self.currentFillType)
	if delta > 0 then
		sprayer:setFillLevel(oldFillLevel + delta, self.currentFillType, true)
		delta = sprayer:getFillLevel(self.currentFillType) - oldFillLevel
		if delta > 0 then
			self:setFillLevel(self.fillLevel - delta,self.currentFillType)
		end
	end

	return delta
end
function ManureLagerPlaceable:getIsActivatable(fillable)
	if not fillable:allowFillType(self.currentFillType, false) or self.fillLevel <= 0 then
		return false;
	end;
	return true
end
function ManureLagerPlaceable:allowFillType(fillType, allowEmptying)
	local allowed = false;
	if self.fillTypes[fillType] then
		if self.currentFillType ~= Fillable.FILLTYPE_UNKNOWN then
			if self.currentFillType ~= fillType then
				if self.fillLevel <= self.capacity*0.05 then
					allowed = true; -- fill level is low enough to be overridden
					if allowEmptying then
						self.fillLevel = 0; -- empty the trailer
					end;
				end;
			else
				allowed = true; -- fill type is the same as the trailer's current fill type
			end;
		else
			allowed = true; -- fillable is empty --> Fillable.FILLTYPE_UNKNOWN
		end;
	end;
	return allowed;
end;
function ManureLagerPlaceable:triggerCallback(triggerId, otherActorId, onEnter, onLeave, onStay, otherShapeId)
	if (onEnter or onLeave) then
		if g_currentMission.player ~= nil and otherActorId == g_currentMission.player.rootNode then
			if onEnter then
				self.playerInRange = true
			else
				self.playerInRange = false
			end
		else
			local vehicle = g_currentMission.nodeToVehicle[otherActorId]
			if vehicle ~= nil then
				if onEnter then
					self.vehiclesInRange[vehicle] = true
				else
					self.vehiclesInRange[vehicle] = nil
				end
			end
			if vehicle ~= nil and vehicle.addFillTrigger ~= nil and vehicle.removeFillTrigger ~= nil then			
				if onEnter then
					vehicle:addFillTrigger(self);
					if vehicle.addReFillTrigger ~= nil then
						vehicle:addReFillTrigger(self);
					end;
				else -- onLeave
					vehicle:removeFillTrigger(self);
					if vehicle.removeReFillTrigger ~= nil then
						vehicle:removeReFillTrigger(self);
					end;
				end;
			end;
		end
	end
end
registerPlaceableType("ManureLagerPlaceable", ManureLagerPlaceable);